// Testing USB Host Mass Storage
//
// This example is in the public domain

#include "USBHost_t36.h"
#include "MassStorage.h"

USBHost myusb;
USBHub hub1(myusb);
USBHub hub2(myusb);
USBHub hub3(myusb);
USBHub hub4(myusb);

USBHIDParser hid1(myusb);
KeyboardController keyboard1(myusb);
MouseController mouse1(myusb);

msController msDrive1(myusb);
//msController msDrive2(myusb);

msSCSICapacity_t msCapacity;
msInquiryResponse_t msInquiry;
msRequestSenseResponse_t msSense;

uint8_t msResult = 0;
uint8_t sectorInBuffer[512];
uint8_t sectorOutBuffer[512];
uint32_t start = 0 , end = 0;

void hexDump(const void *ptr, uint32_t len)
{
  uint32_t  i = 0, j = 0;
  uint8_t   c=0;
  const uint8_t *p = (const uint8_t *)ptr;

  Serial.printf("BYTE      00 01 02 03 04 05 06 07 08 09 0A 0B 0C 0D 0E 0F\n");
  Serial.printf("---------------------------------------------------------\n");
  for(i = 0; i <= (len-1); i+=16) {
   Serial.printf("%4.4x      ",i);
   for(j = 0; j < 16; j++) {
      c = p[i+j];
      Serial.printf("%2.2x ",c);
    }
    Serial.printf("  ");
    for(j = 0; j < 16; j++) {
      c = p[i+j];
      if(c > 31 && c < 127)
        Serial.printf("%c",c);
      else
        Serial.printf(".");
    }
    Serial.println();
  }

}

void setup() {
  while (!Serial) ; // wait for Arduino Serial Monitor

  Serial.printf("USB Mass Storage Testing\n");
  myusb.begin();
  delay(10);

  Serial.printf("\nWaiting for device to become available.\n");
  while(!msDrive1.available());
//  while(!msDrive2.available());

  Serial.printf("\nmsReset()\n");
  msDrive1.msReset();
  delay(10);

  msResult = msDrive1.msGetMaxLun();
  Serial.printf("\nmsGetMaxLun(): %d\n", msResult);

  //-------------------------------------------------------
  Serial.printf("\nmsDeviceInquiry()\n");
  msDrive1.WaitMediaReady();
  msResult = msDrive1.msDeviceInquiry(&msInquiry);
  if(msResult)
    Serial.printf("Failed: %d", msResult);
  else
    Serial.printf("Passed\n"); 
  hexDump(&msInquiry,sizeof(msInquiry)); 

  Serial.printf("Removable Device: ");
  if(msInquiry.Removable == 1)
    Serial.printf("YES\n");
  else
    Serial.printf("NO\n");
  Serial.printf("VendorID: %8.8s\n",msInquiry.VendorID);
  Serial.printf("ProductID: %16.16s\n",msInquiry.ProductID);
  Serial.printf("RevisionID: %4.4s\n",msInquiry.RevisionID);

  // First and Last sector to read/write. 
  uint32_t i, first = 1UL, last = 100UL;

  //-------------------------------------------------------
  Serial.printf("\nmsReadDeviceCapacity()\n");
  msDrive1.WaitMediaReady();
  msResult = msDrive1.msReadDeviceCapacity(&msCapacity);
  if(msResult)
    Serial.printf("Failed: %d", msResult);
  else
    Serial.printf("Passed\n"); 
  Serial.printf("Number of sectors = %d\n",msCapacity.Blocks);
  Serial.printf("Sector size = %d\n",msCapacity.BlockSize);

  //-------------------------------------------------------
  // CAUTION: This will overwrite and destroy any data on the drive!
  // Do not enable if you have data you don't want overwritten.
  // Use a blank formated drive.
#if 0   // <----- Change to '1' to overwrite sectors.

  Serial.println();
  Serial.printf("msWriteBlocks(): \n");
  start = millis();
  msDrive1.WaitMediaReady();
  for(i = first; i<=last; i++) {
    memset(sectorOutBuffer,i&255,512);
    msResult = msDrive1.msWriteBlocks(i,1,512,sectorOutBuffer);
    end = millis()-start;
    if(msResult) {
      Serial.printf("Failed: %d\n", msResult);
    }
  }
  Serial.printf("Took %dms to write %d sectors\n",end, i);
  Serial.printf("%i Sectors Written.\n",i);

#endif

  //-------------------------------------------------------
  Serial.printf("\nmsReadBlocks(): \n");
  start = millis();
  msDrive1.WaitMediaReady();
  for(i = first; i<=last; i++) {
    msResult = msDrive1.msReadBlocks(i,1,512,sectorInBuffer);
    end = millis()-start;
    if(msResult) {
      Serial.printf("Failed: %d\n",msResult);
    }
//    hexDump(sectorInBuffer,sizeof(sectorInBuffer)); 
//    Serial.printf("Sector Number %d\n",i);
  }
  Serial.printf("Took %dms to read %d sectors\n",end, i);
  Serial.printf("%i Sectors Read.\n",i);

  Serial.printf("Done...\n");

  keyboard1.attachPress(OnPress);
}

void loop() {
  myusb.Task();
  if(mouse1.available()) {
    Serial.print("Mouse: buttons = ");
    Serial.print(mouse1.getButtons());
    Serial.print(",  mouseX = ");
    Serial.print(mouse1.getMouseX());
    Serial.print(",  mouseY = ");
    Serial.print(mouse1.getMouseY());
    Serial.print(",  wheel = ");
    Serial.print(mouse1.getWheel());
    Serial.print(",  wheelH = ");
    Serial.print(mouse1.getWheelH());
    Serial.println();
#if 1
    if(mouse1.getButtons() == 1) {
      msDrive1.WaitMediaReady();
      start = millis();
      msResult = msDrive1.msReadBlocks(0,1,512,sectorInBuffer);
      end = millis()-start;
      hexDump(sectorInBuffer,sizeof(sectorInBuffer)); 
      Serial.printf("Took %dms to read sector %d\n",end, 0);
    }
#endif    
    mouse1.mouseDataClear();
  }
  if(!msDrive1.available())
    Serial.printf("Device disconnected...\n");
  while(!msDrive1.available());
}

void OnPress(int key)
{
  Serial.print("key '");
  Serial.print((char)key);
  Serial.print("'  ");
  Serial.println(key);
  //Serial.print("key ");
  //Serial.print((char)keyboard1.getKey());
  //Serial.print("  ");
  //Serial.print((char)keyboard2.getKey());
  //Serial.println();
}
